@@ -96,26 +96,29 @@ assert.EQ(m1$initial,
 c(-0.851594294907422, -0.0107066895370536, -0.226958540075445, 0.0355906625338308,
   0.048010654640958, 0.0847493155436896, 0.0133604488401352, -0.024115201062159,
   0.0270535337324518, 0.146022135657894, -0.00751380783260833, -0.417638086169033)
-          , tol = 1e-13, check.attr=FALSE, giveRE=TRUE)
+          , tol = 1e-13, check.attributes=FALSE, giveRE=TRUE)
 
 ## MM: I'm shocked how much this changes after every tweak ...
 
 (arch <- Sys.info()[["machine"]])
 
+## Something strange going on: R CMD check is different from interactive R, here.
+## ???? [I see that the byte compiler is not listed in sessionInfo]
+## In any case, take the dput(.) output from the *.Rout[.fail] file
 dput(signif(unname(coef(m1)), 11)) ## -->
 beta1 <- list(i686 =
-c(-0.83703411949, 0.0085332123121, -0.16723425506, 0.040966496642,
-  0.042392008333, 0.063163791628, 0.018623119547, -0.0063568893968,
-  0.1143288682, 0.091609212, -0.025109234019, -0.6689999431)
+c(-0.83715700394, 0.0085488694315, -0.16734609346, 0.040965601691, 
+  0.042387113444, 0.063146240793, 0.018632137866, -0.0062886781262, 
+  0.11466679192, 0.091457894347, -0.025009954018, -0.66867971209)
 , "x86_64" =
 c(-0.83723213945, 0.0085385261915, -0.16697112315, 0.040985126003,
   0.042400738973, 0.063168847366, 0.01863253681, -0.0064477807228,
   0.11488937188, 0.091283185006, -0.025627390293, -0.66995658693)
 )
-## just FYI :
-assert.EQ(beta1[[1]], beta1[[2]], tol = 0.002, check.attr=FALSE, giveRE=TRUE)
+## just FYI: difference 32-bit vs 64-bit:
+assert.EQ(beta1[[1]], beta1[[2]], tol = 0.002, check.attributes=FALSE, giveRE=TRUE)
 
-assert.EQ(coef(m1), beta1[[arch]], tol = 1e-10, check.attr=FALSE, giveRE=TRUE)
+assert.EQ(coef(m1), beta1[[arch]], tol = 1e-10, check.attributes=FALSE, giveRE=TRUE)
 
 ## The same, with another seed:
 set.seed(64)
@@ -132,33 +135,22 @@ assert.EQ(m2$initial, ## so this is *not* platform (32bit/64bit) dependent:
 c(-1.204304813829, 0.02776038445201, -0.3680174045842, 0.04325746912892,
   0.03895315289169, 0.04537145479989, 0.02847987541025, 0.07073207523212,
   0.355491639539, 0.1822955449528, 0.1323720331562, -0.3419939094877)
-          , tol = 1e-12, check.attr=FALSE, giveRE=TRUE)
-
-beta2 <-
- c(-0.722466858080995, 0.00853214741170862, -0.167080756780928, 0.0409708489859932,
-   0.0423892852206197, 0.0631575676559838,  0.0186247550404213, -0.114368960856547,
-   -0.12071086045638,  0.0913163131743179, -0.0254084789601603, -0.669355337605894)
-
-beta2 <-
- c(-0.719639958181682, 0.00850921638739351, -0.166172980617748, 0.0410112563931713,
-   0.0423638710960024, 0.0630362342605086, 0.0186359589765251, -0.116645159395719,
- -0.123130115061652, 0.0910865027225399, -0.0256449044169698, -0.67024227284216)
+          , tol = 1e-12, check.attributes=FALSE, giveRE=TRUE)
 
 dput(signif(unname(coef(m2)), 11)) ## -->
 beta2 <- list(i686 =
-c(-0.83731166205, 0.0085694733057, -0.16776715809, 0.040959111008,
-  0.042395767814, 0.063185207067, 0.018635039106, -0.0062832385739,
-  0.11406869258, 0.091362008749, -0.025362102704, -0.6688654014)
+c(-0.83698669149, 0.0085587296184, -0.16778044558, 0.040960021262, 
+  0.042402954975, 0.063188868629, 0.018630275088, -0.0061015509403, 
+  0.11385896307, 0.090966386294, -0.02572887737, -0.66945784056)
 , "x86_64" =
 c(-0.83687097624, 0.0085341676033, -0.1674299545, 0.040968820903,
   0.042397459287, 0.063159075944, 0.018625582804, -0.0063140636571,
   0.11426134017, 0.091317308575, -0.025373078819, -0.66957444238)
 )
+## just FYI: difference 32-bit vs 64-bit:
+assert.EQ(beta2[[1]], beta2[[2]], tol = 0.001, check.attributes=FALSE, giveRE=TRUE)
 
-## just FYI :
-assert.EQ(beta2[[1]], beta2[[2]], tol = 0.001, check.attr=FALSE, giveRE=TRUE)
-
-assert.EQ(coef(m2), beta2[[arch]], tol = 1e-10, check.attr=FALSE, giveRE=TRUE)
+assert.EQ(coef(m2), beta2[[arch]], tol = 1e-10, check.attributes=FALSE, giveRE=TRUE)
 ## slight changes of algorithm often change the above by ~ 4e-4 !!!
 
 ###---- Model Selection -----
